package cn.com.infosec.x509.util;

import cn.com.infosec.asn1.ASN1InputStream;
import cn.com.infosec.asn1.x509.CertificatePair;
import cn.com.infosec.asn1.x509.X509CertificateStructure;
import cn.com.infosec.jce.X509LDAPCertStoreParameters;
import cn.com.infosec.jce.provider.X509AttrCertParser;
import cn.com.infosec.jce.provider.X509CRLParser;
import cn.com.infosec.jce.provider.X509CertPairParser;
import cn.com.infosec.jce.provider.X509CertParser;
import cn.com.infosec.x509.X509AttributeCertStoreSelector;
import cn.com.infosec.x509.X509AttributeCertificate;
import cn.com.infosec.x509.X509CRLStoreSelector;
import cn.com.infosec.x509.X509CertPairStoreSelector;
import cn.com.infosec.x509.X509CertStoreSelector;
import cn.com.infosec.x509.X509CertificatePair;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.security.Principal;
import java.security.cert.CertificateParsingException;
import java.security.cert.X509CRL;
import java.security.cert.X509Certificate;
import java.sql.Date;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;
import javax.security.auth.x500.X500Principal;

public class LDAPStoreHelper {
   private static String LDAP_PROVIDER;
   private static String REFERRALS_IGNORE;
   private static final String SEARCH_SECURITY_LEVEL = "none";
   private static final String URL_CONTEXT_PREFIX = "com.sun.jndi.url";
   private static int cacheSize;
   private static long lifeTime;
   private X509LDAPCertStoreParameters params;
   private Map cacheMap;

   public LDAPStoreHelper(X509LDAPCertStoreParameters params) {
      this.cacheMap = new HashMap(cacheSize);
      this.params = params;
   }

   private DirContext connectLDAP() {
      Properties var1;
      Properties var10000 = var1 = new Properties;
      var1.<init>();
      var1.setProperty("java.naming.factory.initial", LDAP_PROVIDER);
      var1.setProperty("java.naming.batchsize", "0");
      var10000.setProperty("java.naming.provider.url", this.params.getLdapURL());
      var10000.setProperty("java.naming.factory.url.pkgs", "com.sun.jndi.url");
      var10000.setProperty("java.naming.referral", REFERRALS_IGNORE);
      var10000.setProperty("java.naming.security.authentication", "none");
      return new InitialDirContext(var1);
   }

   private String parseDN(String subject, String dNAttributeName) {
      int var3;
      if ((var3 = subject.toLowerCase().indexOf(dNAttributeName.toLowerCase() + "=")) == -1) {
         return "";
      } else {
         String var4;
         int subject1;
         if ((subject1 = (var4 = subject.substring(var3 + dNAttributeName.length())).indexOf(44)) == -1) {
            subject1 = var4.length();
         }

         while(var4.charAt(subject1 - 1) == '\\') {
            if ((subject1 = var4.indexOf(44, subject1 + 1)) == -1) {
               subject1 = var4.length();
            }
         }

         String var10000 = var4.substring(0, subject1);
         if ((var4 = var10000.substring(var10000.indexOf(61) + 1)).charAt(0) == ' ') {
            var4 = var4.substring(1);
         }

         if (var4.startsWith("\"")) {
            var4 = var4.substring(1);
         }

         if (var4.endsWith("\"")) {
            var4 = var4.substring(0, var4.length() - 1);
         }

         return var4;
      }
   }

   private Set createCerts(List list, X509CertStoreSelector xselector) {
      HashSet var8;
      var8 = new HashSet.<init>();
      Iterator list1 = list.iterator();
      X509CertParser var3;
      var3 = new X509CertParser.<init>();

      while(list1.hasNext()) {
         X509CertStoreSelector var10000;
         boolean var10001;
         X509Certificate var11;
         try {
            var10000 = xselector;
            var3.engineInit(new ByteArrayInputStream((byte[])list1.next()));
            var11 = (X509Certificate)var3.engineRead();
         } catch (Exception var7) {
            var10001 = false;
            continue;
         }

         X509Certificate var4 = var11;

         boolean var10;
         try {
            var10 = var10000.match(var11);
         } catch (Exception var6) {
            var10001 = false;
            continue;
         }

         if (var10) {
            try {
               var8.add(var4);
            } catch (Exception var5) {
               var10001 = false;
            }
         }
      }

      return var8;
   }

   private List certSubjectSerialSearch(X509CertStoreSelector xselector, String[] attrs, String[] attrNames, String[] subjectAttributeNames) {
      ArrayList var5;
      var5 = new ArrayList.<init>();
      String var6 = null;
      String var7 = this.getSubjectAsString(xselector);
      if (xselector.getSerialNumber() != null) {
         var6 = xselector.getSerialNumber().toString();
      }

      if (xselector.getCertificate() != null) {
         var7 = xselector.getCertificate().getSubjectX500Principal().getName("RFC1779");
         var6 = xselector.getCertificate().getSerialNumber().toString();
      }

      if (var7 != null) {
         for(int xselector1 = 0; xselector1 < subjectAttributeNames.length; ++xselector1) {
            String var8 = this.parseDN(var7, subjectAttributeNames[xselector1]);
            var5.addAll(this.search(attrNames, "*" + var8 + "*", attrs));
         }
      }

      if (var6 != null && this.params.getSearchForSerialNumberIn() != null) {
         var5.addAll(this.search(this.splitString(this.params.getSearchForSerialNumberIn()), var6, attrs));
      }

      if (var6 == null && var7 == null) {
         var5.addAll(this.search(attrNames, "*", attrs));
      }

      return var5;
   }

   private List crossCertificatePairSubjectSearch(X509CertPairStoreSelector xselector, String[] attrs, String[] attrNames, String[] subjectAttributeNames) {
      ArrayList var5;
      var5 = new ArrayList.<init>();
      String var6 = null;
      if (xselector.getForwardSelector() != null) {
         var6 = this.getSubjectAsString(xselector.getForwardSelector());
      }

      if (xselector.getCertPair() != null && xselector.getCertPair().getForward() != null) {
         var6 = xselector.getCertPair().getForward().getSubjectX500Principal().getName("RFC1779");
      }

      if (var6 != null) {
         for(int xselector1 = 0; xselector1 < subjectAttributeNames.length; ++xselector1) {
            String var7 = this.parseDN(var6, subjectAttributeNames[xselector1]);
            var5.addAll(this.search(attrNames, "*" + var7 + "*", attrs));
         }
      }

      if (var6 == null) {
         var5.addAll(this.search(attrNames, "*", attrs));
      }

      return var5;
   }

   private List attrCertSubjectSerialSearch(X509AttributeCertStoreSelector xselector, String[] attrs, String[] attrNames, String[] subjectAttributeNames) {
      ArrayList var5;
      var5 = new ArrayList.<init>();
      String var6 = null;
      HashSet var7;
      var7 = new HashSet.<init>();
      Principal[] var8 = null;
      if (xselector.getHolder() != null) {
         if (xselector.getHolder().getSerialNumber() != null) {
            var7.add(xselector.getHolder().getSerialNumber().toString());
         }

         if (xselector.getHolder().getEntityNames() != null) {
            var8 = xselector.getHolder().getEntityNames();
         }
      }

      if (xselector.getAttributeCert() != null) {
         if (xselector.getAttributeCert().getHolder().getEntityNames() != null) {
            var8 = xselector.getAttributeCert().getHolder().getEntityNames();
         }

         var7.add(xselector.getAttributeCert().getSerialNumber().toString());
      }

      if (var8 != null) {
         if (var8[0] instanceof X500Principal) {
            var6 = ((X500Principal)var8[0]).getName("RFC1779");
         } else {
            var6 = var8[0].getName();
         }
      }

      if (xselector.getSerialNumber() != null) {
         var7.add(xselector.getSerialNumber().toString());
      }

      if (var6 != null) {
         for(int xselector1 = 0; xselector1 < subjectAttributeNames.length; ++xselector1) {
            String var12 = this.parseDN(var6, subjectAttributeNames[xselector1]);
            var5.addAll(this.search(attrNames, "*" + var12 + "*", attrs));
         }
      }

      if (var7.size() > 0 && this.params.getSearchForSerialNumberIn() != null) {
         Iterator xselector2 = var7.iterator();

         while(xselector2.hasNext()) {
            String subjectAttributeNames1 = (String)xselector2.next();
            var5.addAll(this.search(this.splitString(this.params.getSearchForSerialNumberIn()), subjectAttributeNames1, attrs));
         }
      }

      if (var7.size() == 0 && var6 == null) {
         var5.addAll(this.search(attrNames, "*", attrs));
      }

      return var5;
   }

   private List cRLIssuerSearch(X509CRLStoreSelector xselector, String[] attrs, String[] attrNames, String[] issuerAttributeNames) {
      ArrayList var5;
      var5 = new ArrayList.<init>();
      String var6 = null;
      HashSet var7;
      var7 = new HashSet.<init>();
      if (xselector.getIssuers() != null) {
         var7.addAll(xselector.getIssuers());
      }

      if (xselector.getCertificateChecking() != null) {
         var7.add(this.getCertificateIssuer(xselector.getCertificateChecking()));
      }

      if (xselector.getAttrCertificateChecking() != null) {
         Principal[] xselector1 = xselector.getAttrCertificateChecking().getIssuer().getPrincipals();

         for(int var8 = 0; var8 < xselector1.length; ++var8) {
            if (xselector1[var8] instanceof X500Principal) {
               var7.add(xselector1[var8]);
            }
         }
      }

      Iterator xselector2 = var7.iterator();

      while(xselector2.hasNext()) {
         var6 = ((X500Principal)xselector2.next()).getName("RFC1779");

         for(int var11 = 0; var11 < issuerAttributeNames.length; ++var11) {
            String var12 = this.parseDN(var6, issuerAttributeNames[var11]);
            var5.addAll(this.search(attrNames, "*" + var12 + "*", attrs));
         }
      }

      if (var6 == null) {
         var5.addAll(this.search(attrNames, "*", attrs));
      }

      return var5;
   }

   private List search(String[] param1, String param2, String[] param3) {
      // $FF: Couldn't be decompiled
   }

   private Set createCRLs(List list, X509CRLStoreSelector xselector) {
      List var10000 = list;
      HashSet var8;
      var8 = new HashSet.<init>();
      X509CRLParser list1;
      list1 = new X509CRLParser.<init>();
      Iterator var3 = var10000.iterator();

      while(var3.hasNext()) {
         boolean var10001;
         X509CRLStoreSelector var10;
         X509CRL var12;
         try {
            var10 = xselector;
            list1.engineInit(new ByteArrayInputStream((byte[])var3.next()));
            var12 = (X509CRL)list1.engineRead();
         } catch (StreamParsingException var7) {
            var10001 = false;
            continue;
         }

         X509CRL var4 = var12;

         boolean var11;
         try {
            var11 = var10.match(var12);
         } catch (StreamParsingException var6) {
            var10001 = false;
            continue;
         }

         if (var11) {
            try {
               var8.add(var4);
            } catch (StreamParsingException var5) {
               var10001 = false;
            }
         }
      }

      return var8;
   }

   private Set createCrossCertificatePairs(List list, X509CertPairStoreSelector xselector) {
      HashSet var23;
      var23 = new HashSet.<init>();

      int var5;
      for(int var3 = 0; var3 < list.size(); var3 = var5 + 1) {
         X509CertificatePair var7;
         boolean var10001;
         label96: {
            X509CertificatePair var26;
            label95: {
               label94: {
                  label106: {
                     try {
                        try {
                           X509CertPairParser var27 = new X509CertPairParser();
                           var27.engineInit(new ByteArrayInputStream((byte[])list.get(var3)));
                           var26 = (X509CertificatePair)var27.engineRead();
                           break label95;
                        } catch (StreamParsingException var20) {
                        }
                     } catch (CertificateParsingException var21) {
                        var10001 = false;
                        break label94;
                     } catch (IOException var22) {
                        var10001 = false;
                        break label106;
                     }

                     List var10000;
                     byte[] var29;
                     try {
                        var10000 = list;
                        var29 = (byte[])list.get(var3);
                     } catch (CertificateParsingException var18) {
                        var10001 = false;
                        break label94;
                     } catch (IOException var19) {
                        var10001 = false;
                        break label106;
                     }

                     byte[] var4 = var29;
                     int var30 = var5 = var3 + 1;

                     byte[] var25;
                     try {
                        var25 = (byte[])var10000.get(var30);
                     } catch (CertificateParsingException var16) {
                        var10001 = false;
                        break label94;
                     } catch (IOException var17) {
                        var10001 = false;
                        break label106;
                     }

                     byte[] var6 = var25;

                     try {
                        var26 = new X509CertificatePair;
                     } catch (CertificateParsingException var14) {
                        var10001 = false;
                        break label94;
                     } catch (IOException var15) {
                        var10001 = false;
                        break label106;
                     }

                     var7 = var26;

                     try {
                        var26.<init>(new CertificatePair(X509CertificateStructure.getInstance((new ASN1InputStream(var4)).readObject()), X509CertificateStructure.getInstance((new ASN1InputStream(var6)).readObject())));
                        break label96;
                     } catch (CertificateParsingException var12) {
                        var10001 = false;
                        break label94;
                     } catch (IOException var13) {
                        var10001 = false;
                     }
                  }

                  var5 = var3;
                  continue;
               }

               var5 = var3;
               continue;
            }

            X509CertificatePair var24 = var26;
            var5 = var3;
            var7 = var24;
         }

         boolean var28;
         try {
            var28 = xselector.match(var7);
         } catch (CertificateParsingException var10) {
            var10001 = false;
            continue;
         } catch (IOException var11) {
            var10001 = false;
            continue;
         }

         if (var28) {
            try {
               var23.add(var7);
            } catch (CertificateParsingException var8) {
               var10001 = false;
            } catch (IOException var9) {
               var10001 = false;
            }
         }
      }

      return var23;
   }

   private Set createAttributeCertificates(List list, X509AttributeCertStoreSelector xselector) {
      HashSet var8;
      var8 = new HashSet.<init>();
      Iterator list1 = list.iterator();
      X509AttrCertParser var3;
      var3 = new X509AttrCertParser.<init>();

      while(list1.hasNext()) {
         X509AttributeCertStoreSelector var10000;
         boolean var10001;
         X509AttributeCertificate var11;
         try {
            var10000 = xselector;
            var3.engineInit(new ByteArrayInputStream((byte[])list1.next()));
            var11 = (X509AttributeCertificate)var3.engineRead();
         } catch (StreamParsingException var7) {
            var10001 = false;
            continue;
         }

         X509AttributeCertificate var4 = var11;

         boolean var10;
         try {
            var10 = var10000.match(var11);
         } catch (StreamParsingException var6) {
            var10001 = false;
            continue;
         }

         if (var10) {
            try {
               var8.add(var4);
            } catch (StreamParsingException var5) {
               var10001 = false;
            }
         }
      }

      return var8;
   }

   private synchronized void addToCache(String searchCriteria, List list) {
      Date var3;
      var3 = new Date.<init>(System.currentTimeMillis());
      ArrayList var4;
      ArrayList var10001 = var4 = new ArrayList;
      var4.<init>();
      var4.add(var3);
      var10001.add(list);
      if (this.cacheMap.containsKey(searchCriteria)) {
         this.cacheMap.put(searchCriteria, var4);
      } else {
         if (this.cacheMap.size() >= cacheSize) {
            Iterator list1 = this.cacheMap.entrySet().iterator();
            long var5 = var3.getTime();
            Object var11 = null;

            while(list1.hasNext()) {
               Map.Entry var7;
               long var8;
               if ((var8 = ((Date)((List)(var7 = (Map.Entry)list1.next()).getValue()).get(0)).getTime()) < var5) {
                  var11 = var7.getKey();
                  var5 = var8;
               }
            }

            this.cacheMap.remove(var11);
         }

         this.cacheMap.put(searchCriteria, var4);
      }

   }

   private List getFromCache(String searchCriteria) {
      List var4;
      List var10000 = var4 = (List)this.cacheMap.get(searchCriteria);
      long var2 = System.currentTimeMillis();
      if (var10000 != null) {
         return ((Date)var4.get(0)).getTime() < var2 - lifeTime ? null : (List)var4.get(1);
      } else {
         return null;
      }
   }

   private String[] splitString(String str) {
      return str.split("\\s+");
   }

   private String getSubjectAsString(X509CertStoreSelector param1) {
      // $FF: Couldn't be decompiled
   }

   private X500Principal getCertificateIssuer(X509Certificate cert) {
      return cert.getIssuerX500Principal();
   }

   public Collection getAuthorityRevocationLists(X509CRLStoreSelector selector) {
      String[] var2 = this.splitString(this.params.getAuthorityRevocationListAttribute());
      String[] var3 = this.splitString(this.params.getLdapAuthorityRevocationListAttributeName());
      String[] var4 = this.splitString(this.params.getAuthorityRevocationListIssuerAttributeName());
      Set var5;
      if ((var5 = this.createCRLs(this.cRLIssuerSearch(selector, var2, var3, var4), selector)).size() == 0) {
         var5.addAll(this.createCRLs(this.cRLIssuerSearch(new X509CRLStoreSelector(), var2, var3, var4), selector));
      }

      return var5;
   }

   public Collection getAttributeCertificateRevocationLists(X509CRLStoreSelector selector) {
      String[] var2 = this.splitString(this.params.getAttributeCertificateRevocationListAttribute());
      String[] var3 = this.splitString(this.params.getLdapAttributeCertificateRevocationListAttributeName());
      String[] var4 = this.splitString(this.params.getAttributeCertificateRevocationListIssuerAttributeName());
      Set var5;
      if ((var5 = this.createCRLs(this.cRLIssuerSearch(selector, var2, var3, var4), selector)).size() == 0) {
         var5.addAll(this.createCRLs(this.cRLIssuerSearch(new X509CRLStoreSelector(), var2, var3, var4), selector));
      }

      return var5;
   }

   public Collection getAttributeAuthorityRevocationLists(X509CRLStoreSelector selector) {
      String[] var2 = this.splitString(this.params.getAttributeAuthorityRevocationListAttribute());
      String[] var3 = this.splitString(this.params.getLdapAttributeAuthorityRevocationListAttributeName());
      String[] var4 = this.splitString(this.params.getAttributeAuthorityRevocationListIssuerAttributeName());
      Set var5;
      if ((var5 = this.createCRLs(this.cRLIssuerSearch(selector, var2, var3, var4), selector)).size() == 0) {
         var5.addAll(this.createCRLs(this.cRLIssuerSearch(new X509CRLStoreSelector(), var2, var3, var4), selector));
      }

      return var5;
   }

   public Collection getCrossCertificatePairs(X509CertPairStoreSelector selector) {
      String[] var2 = this.splitString(this.params.getCrossCertificateAttribute());
      String[] var3 = this.splitString(this.params.getLdapCrossCertificateAttributeName());
      String[] var4 = this.splitString(this.params.getCrossCertificateSubjectAttributeName());
      Set var5;
      if ((var5 = this.createCrossCertificatePairs(this.crossCertificatePairSubjectSearch(selector, var2, var3, var4), selector)).size() == 0) {
         LDAPStoreHelper var10001 = this;
         LDAPStoreHelper var10002 = this;
         X509CertStoreSelector var7;
         var7 = new X509CertStoreSelector.<init>();
         X509CertPairStoreSelector var6;
         X509CertPairStoreSelector var10003 = var6 = new X509CertPairStoreSelector;
         var6.<init>();
         var6.setForwardSelector(var7);
         var10003.setReverseSelector(var7);
         var5.addAll(var10001.createCrossCertificatePairs(var10002.crossCertificatePairSubjectSearch(var10003, var2, var3, var4), selector));
      }

      return var5;
   }

   public Collection getUserCertificates(X509CertStoreSelector selector) {
      String[] var2 = this.splitString(this.params.getUserCertificateAttribute());
      String[] var3 = this.splitString(this.params.getLdapUserCertificateAttributeName());
      String[] var4 = this.splitString(this.params.getUserCertificateSubjectAttributeName());
      Set var5;
      if ((var5 = this.createCerts(this.certSubjectSerialSearch(selector, var2, var3, var4), selector)).size() == 0) {
         var5.addAll(this.createCerts(this.certSubjectSerialSearch(new X509CertStoreSelector(), var2, var3, var4), selector));
      }

      return var5;
   }

   public Collection getAACertificates(X509AttributeCertStoreSelector selector) {
      String[] var2 = this.splitString(this.params.getAACertificateAttribute());
      String[] var3 = this.splitString(this.params.getLdapAACertificateAttributeName());
      String[] var4 = this.splitString(this.params.getAACertificateSubjectAttributeName());
      Set var5;
      if ((var5 = this.createAttributeCertificates(this.attrCertSubjectSerialSearch(selector, var2, var3, var4), selector)).size() == 0) {
         var5.addAll(this.createAttributeCertificates(this.attrCertSubjectSerialSearch(new X509AttributeCertStoreSelector(), var2, var3, var4), selector));
      }

      return var5;
   }

   public Collection getAttributeDescriptorCertificates(X509AttributeCertStoreSelector selector) {
      String[] var2 = this.splitString(this.params.getAttributeDescriptorCertificateAttribute());
      String[] var3 = this.splitString(this.params.getLdapAttributeDescriptorCertificateAttributeName());
      String[] var4 = this.splitString(this.params.getAttributeDescriptorCertificateSubjectAttributeName());
      Set var5;
      if ((var5 = this.createAttributeCertificates(this.attrCertSubjectSerialSearch(selector, var2, var3, var4), selector)).size() == 0) {
         var5.addAll(this.createAttributeCertificates(this.attrCertSubjectSerialSearch(new X509AttributeCertStoreSelector(), var2, var3, var4), selector));
      }

      return var5;
   }

   public Collection getCACertificates(X509CertStoreSelector selector) {
      String[] var2 = this.splitString(this.params.getCACertificateAttribute());
      String[] var3 = this.splitString(this.params.getLdapCACertificateAttributeName());
      String[] var4 = this.splitString(this.params.getCACertificateSubjectAttributeName());
      Set var5;
      if ((var5 = this.createCerts(this.certSubjectSerialSearch(selector, var2, var3, var4), selector)).size() == 0) {
         var5.addAll(this.createCerts(this.certSubjectSerialSearch(new X509CertStoreSelector(), var2, var3, var4), selector));
      }

      return var5;
   }

   public Collection getDeltaCertificateRevocationLists(X509CRLStoreSelector selector) {
      String[] var2 = this.splitString(this.params.getDeltaRevocationListAttribute());
      String[] var3 = this.splitString(this.params.getLdapDeltaRevocationListAttributeName());
      String[] var4 = this.splitString(this.params.getDeltaRevocationListIssuerAttributeName());
      Set var5;
      if ((var5 = this.createCRLs(this.cRLIssuerSearch(selector, var2, var3, var4), selector)).size() == 0) {
         var5.addAll(this.createCRLs(this.cRLIssuerSearch(new X509CRLStoreSelector(), var2, var3, var4), selector));
      }

      return var5;
   }

   public Collection getAttributeCertificateAttributes(X509AttributeCertStoreSelector selector) {
      String[] var2 = this.splitString(this.params.getAttributeCertificateAttributeAttribute());
      String[] var3 = this.splitString(this.params.getLdapAttributeCertificateAttributeAttributeName());
      String[] var4 = this.splitString(this.params.getAttributeCertificateAttributeSubjectAttributeName());
      Set var5;
      if ((var5 = this.createAttributeCertificates(this.attrCertSubjectSerialSearch(selector, var2, var3, var4), selector)).size() == 0) {
         var5.addAll(this.createAttributeCertificates(this.attrCertSubjectSerialSearch(new X509AttributeCertStoreSelector(), var2, var3, var4), selector));
      }

      return var5;
   }

   public Collection getCertificateRevocationLists(X509CRLStoreSelector selector) {
      String[] var2 = this.splitString(this.params.getCertificateRevocationListAttribute());
      String[] var3 = this.splitString(this.params.getLdapCertificateRevocationListAttributeName());
      String[] var4 = this.splitString(this.params.getCertificateRevocationListIssuerAttributeName());
      Set var5;
      if ((var5 = this.createCRLs(this.cRLIssuerSearch(selector, var2, var3, var4), selector)).size() == 0) {
         var5.addAll(this.createCRLs(this.cRLIssuerSearch(new X509CRLStoreSelector(), var2, var3, var4), selector));
      }

      return var5;
   }
}
